using System;
using System.Reflection;
using System.Reflection.Emit;
using Micro.Interfaces;
using Micro.Interfaces.SymTab;

namespace Micro.Implementation
{
    public class CodeGen : ICodeGen
    {
        // metadata builder for the program assembly
        private AssemblyBuilder assembly;
        // metadata builder for the program module
        private ModuleBuilder module;
        // metadata builder for the main class
        private TypeBuilder program;
        // IL stream of currently compiled method
        private ILGenerator il;
        // needed for the readi method (important to preserve the character successing a number,
        private IError fError;


        public CodeGen(string target, IError err)
        {
            fError = err;
            AssemblyName assemblyName = new AssemblyName();
            assemblyName.Name = target;

            assembly = AppDomain.CurrentDomain.DefineDynamicAssembly(
                assemblyName, AssemblyBuilderAccess.Save);
            module = assembly.DefineDynamicModule(target + "Module",
                                                  target + ".exe");
            program = module.DefineType(target, TypeAttributes.Class |
                                                TypeAttributes.Public);
        }

        public void CreateMetadata(ISymbol sym)
        {
            // TODO
        }

        // ---------- instruction generation

        public void Load(IItem x)
        {
            // TODO
        }

        public void Assign(IItem x, IItem y)
        {
            // make sure y is loaded (if already on stack, nothing will happen)
            // TODO
        }

        public void Jump(OpCode condition, Label label)
        {
            // TODO
        }

        public void Jump(IItem x)
        {
            // TODO
        }

        public void Emit(OpCode opcode)
        {
            il.Emit(opcode);
        }

        public void EmitCall(MethodInfo meth)
        {
            il.EmitCall(OpCodes.Call, meth, null);
        }

        public void EmitCall(ISymbol function)
        {
            if (function.Kind != SymbolKind.Func)
            {
                throw new ArgumentException(
                    "function must be of kind Func to emit a call");
            }
            EmitCall(function.Meth);
        }

        public IItem CreateItem(int val, IStruct type)
        {
            Item item = new Item();
            item.Kind = ItemKind.Const;
            item.Val = val;
            item.Type = type;
            return item;
        }

        public IItem CreateItem(ISymbol sym)
        {
            // TODO
            return null;
        }

        public IItem CreateItem(TokenKind relop)
        {
            // TODO
            return null;
        }

        public Label CreateLabel()
        {
            return il.DefineLabel();
        }

        public void MarkLabel(Label label)
        {
            il.MarkLabel(label);
        }

        public void WritePEFile()
        {
            program.CreateType();
            if (assembly.EntryPoint == null)
            {
                fError.Error("Missing 'main' function");
            }
            assembly.Save(assembly.GetName().Name + ".exe");
        }


        // An item stores the attributes of an operand during code generation.
        private class Item : IItem
        {
            private ItemKind kind;
            private IStruct type;
            private int val;
            private int adr;
            private TokenKind relop;
            private ISymbol sym;
            private Label label;

            // Const, Local, Static, Stack, Field, Elem, Method, Cond
            public ItemKind Kind
            {
                get { return kind; }
                set { kind = value; }
            }

            // item type
            public IStruct Type
            {
                get { return type; }
                set { type = value; }
            }

            // Const: value
            public int Val
            {
                get { return val; }
                set { val = value; }
            }

            // Arg, Local: offset
            public int Adr
            {
                get { return adr; }
                set { adr = value; }
            }

            // Cond: token code of relational operator
            public TokenKind Relop
            {
                get { return relop; }
                set { relop = value; }
            }

            // Field, Func: node from symbol table
            public ISymbol Sym
            {
                get { return sym; }
                set { sym = value; }
            }

            // Cond: target
            public Label Label
            {
                get { return label; }
                set { label = value; }
            }
        }
    }
}