using System;
using Micro.Interfaces;
using Micro.Interfaces.SymTab;

namespace Micro.Implementation
{
    public class SymTab : ISymTab
    {
        #region private classes Scope and Struct
        private class Scope
        {
            public Scope Outer;
            public ISymbol Locals;
            public int NArgs;
            public int NLocs;
        }
        
        /// A type structure stores the type attributes of a declared type.
        private class Struct : IStruct
        {
            private StructKind fKind;

            // this field is necessary for the building the
            // assembly with System.Reflection.Emit
            // CLR runtime type object
            private Type sysType;

            public Struct(StructKind kind)
            {
                fKind = kind;
                // set CLR type of Struct instance
                if (StructKind.Int == kind)
                {
                    sysType = typeof(int);
                }
                else
                {
                    sysType = typeof(char);
                }
            }

            public override bool Equals(object o)
            {
                return this == o;
            }

            public override int GetHashCode()
            {
                return (int)Kind;
            }

            public StructKind Kind
            {
                get { return fKind; }
            }

            public Type SysType
            {
                get { return sysType; }
            }
        }

        #endregion

        private static readonly IStruct
            intType = new Struct(StructKind.Int),
            charType = new Struct(StructKind.Char);

        // current scope
        private Scope topScope;
        private IError fError;

        public SymTab(IError err)
        {
            fError = err;

            // set up "universe" (= predefined names)
            topScope = new Scope();

            Insert(SymbolKind.Type, "int", intType);
            Insert(SymbolKind.Type, "char", charType);
        }

        public void OpenScope()
        {
            Scope s = new Scope();
            s.NArgs = 0;
            s.NLocs = 0;
            s.Outer = topScope;
            topScope = s;
        }

        public void CloseScope()
        {
            topScope = topScope.Outer;
        }

        public ISymbol Insert(SymbolKind kind, string name, IStruct type)
        {
            // TODO
            return null;
        }
        
        public ISymbol Insert(string name, IStruct type, int value)
        {
            // TODO
            return null;
        }

        public ISymbol FindType(string name)
        {
            // TODO
            return null;
        }

        public ISymbol FindFunc(string name)
        {
            // TODO
            return null;
        }

        public ISymbol FindVar(string name)
        {
            // TODO
            return null;
        }
        
        public void SetLocals(ISymbol function)
        {
            // TODO
        }
    }
}