using System;
using System.IO;
using Micro.Implementation;
using Micro.Interfaces;

namespace Micro
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length != 1)
            {
                PrintHelp();
                return;
            }

            String src = args[0];
            
            if (!src.EndsWith("mic")) {
                PrintHelp();
                return;
            }
            
            String target = Path.GetFileNameWithoutExtension(src);
            
            IError err = new PanicError();
            IScanner scan = new Scanner(new FileStream(src, FileMode.Open, FileAccess.Read), err);
            ICodeGen code = new CodeGen(target, err);
            IParser parse = new Parser(scan, err, code);
            try {
                parse.Parse();
                code.WritePEFile();
            } catch (PanicException panic) {
                Console.WriteLine(panic.Message);
            }
        }

        private static void PrintHelp()
        {
            Console.WriteLine("Syntax: Micro.exe <micro source file name>");
            Console.WriteLine();
            Console.WriteLine("Micro source file names must end with \".mic\".");
        }
    }
}
